/// <reference types="node" />
import { CookieJar } from 'tough-cookie';
import { Middleware, Composed } from 'throwback';
import { Agent } from 'http';
import { SecureContext } from 'tls';
import { Request, Response, ResponseOptions } from 'servie';
import { Body } from 'servie/dist/body/node';
import { NormalizeRequestOptions } from '../common';
/**
 * Extend response with URL.
 */
export interface HttpResponseOptions extends ResponseOptions {
    url: string;
    body: Body;
    httpVersion: string;
}
/**
 * HTTP responses implement a node.js body.
 */
export declare class HttpResponse extends Response implements HttpResponseOptions {
    url: string;
    body: Body;
    httpVersion: string;
    constructor(options: HttpResponseOptions);
}
export declare class Http2Response extends HttpResponse {
}
/**
 * Cookie middleware configuration.
 */
export interface CookieOptions {
    jar: CookieJar;
}
/**
 * Read cookies from the cookie jar.
 */
export declare function getCookies<T extends Request, U extends Response>(options: CookieOptions): Middleware<T, U>;
/**
 * Save response cookies into the cookie jar.
 */
export declare function saveCookies<T extends Request, U extends Response>(options: CookieOptions): Middleware<T, U>;
/**
 * Automatically support decoding zipped responses.
 */
export declare function autoUnzip<T extends Request, U extends HttpResponse>(): Middleware<T, U>;
/**
 * Override the default user-agent header.
 */
export interface NormalizeUserAgentOptions {
    userAgent?: string;
}
/**
 * Set default user-agent in node.js.
 */
export declare function normalizeUserAgent<T extends Request, U extends Response>(options?: NormalizeUserAgentOptions): Middleware<T, U>;
/**
 * Redirect middleware configuration.
 */
export interface FollowRedirectsOptions {
    maxRedirects?: number;
    confirmRedirect?: (request: Request, response: Response) => boolean;
}
/**
 * Middleware function for following HTTP redirects.
 */
export declare function followRedirects<T extends Request, U extends Response>(process: Composed<T, U>, options?: FollowRedirectsOptions): Composed<T, U>;
/**
 * Track HTTP connections for reuse.
 */
export declare class ConnectionManager<T> {
    connections: Map<string, T>;
    get(key: string): T | undefined;
    set(key: string, connection: T): T;
    delete(key: string, connection: T): T;
}
export interface ConcurrencyConnectionManagerOptions {
    maxConnections?: number;
    maxFreeConnections?: number;
}
export interface ConnectionSet<T> {
    used?: Set<T>;
    free?: Set<T>;
    pend?: Array<(connection?: T) => void>;
}
/**
 * Manage HTTP connection reuse.
 */
export declare class ConcurrencyConnectionManager<T> extends ConnectionManager<ConnectionSet<T>> {
    protected options: ConcurrencyConnectionManagerOptions;
    maxConnections: number;
    maxFreeConnections: number;
    constructor(options?: ConcurrencyConnectionManagerOptions);
    /**
     * Create a new connection.
     */
    ready(key: string, onReady: (existingConnection?: T) => void): void;
    getUsedConnection(key: string): T | undefined;
    getFreeConnection(key: string): T | undefined;
    use(key: string, connection: T): void;
    freed(key: string, connection: T, discard: () => void): void;
    remove(key: string, connection: T): void;
}
/**
 * Configure HTTP version negotiation.
 */
export declare enum NegotiateHttpVersion {
    HTTP1_ONLY = 0,
    HTTP2_FOR_HTTPS = 1,
    HTTP2_ONLY = 2
}
/**
 * Node.js HTTP request options.
 */
export interface ForwardOptions {
    keepAlive?: number;
    servername?: string;
    rejectUnauthorized?: boolean;
    ca?: string | Buffer | Array<string | Buffer>;
    cert?: string | Buffer;
    key?: string | Buffer;
    agent?: Agent;
    secureContext?: SecureContext;
    secureProtocol?: string;
    negotiateHttpVersion?: NegotiateHttpVersion;
}
/**
 * Forward request over `http`.
 */
export declare function forward(options: ForwardOptions): (req: Request) => Promise<HttpResponse>;
/**
 * Node.js HTTP transport configuration.
 */
export interface TransportOptions extends ForwardOptions, FollowRedirectsOptions, ForwardOptions, NormalizeRequestOptions, NormalizeUserAgentOptions {
    jar?: CookieJar | false;
    unzip?: false;
    follow?: false;
}
/**
 * Create a request transport using node.js `http` libraries.
 */
export declare function transport(options?: TransportOptions): (req: Request) => Promise<HttpResponse>;
